# -*- coding: utf-8 -*-
#
# Copyright (C) 2006-2007 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://posterity.edgewall.org/wiki/License.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://posterity.edgewall.org/log/.

import logging
import logging.handlers
import os

from pkg_resources import iter_entry_points
from paste.httpexceptions import HTTPSeeOther
from paste.registry import RegistryManager
from paste.urlparser import make_pkg_resources
import selector
from sqlalchemy import create_engine
from elixir import metadata, objectstore

from posterity import app, config, instance, request, response, log
from posterity.component import Component, ComponentManager, Interface, \
        ExtensionPoint
from posterity.util import Config
from posterity.util.i18n import setup_i18n
from posterity.web import Request, Response
from posterity.web.middleware import CSRFMiddleware, SessionMiddleware, \
        TransactionMiddleware, PatchEnvironMiddleware, FormRewriteMiddleware

log_level_map = {
    'CRITICAL': logging.CRITICAL, 'ERROR': logging.ERROR,
    'WARNING': logging.WARNING, 'INFO': logging.INFO,
    'DEBUG': logging.DEBUG
}

# FIXME: move these interfaces somewhere else
class IRoutesProvider(Interface):
    """Url routes provider
    
    Called on application startup allowing components to register their routes
    """
    def add_routes(routes):
        """Register url routes"""


class ISettingsPanel(Interface):
    """Extension point interface for components that need to extend the
    settings panel.
    """
    def get_settings_panels():
        """Return information about additional settings panels
        
        Format: Sequence of {'label': 'My panel', 'href:'/somewhere/'}
        """


class IMessageFilter(Interface):
    """Extension point interface for components that require notification
    of incoming and outgoing messages.
    """
    def incoming_message(self, account, msg):
        """Called when a new message has been added to an account"""

    def outgoing_message(self, account, msg):
        """Called when a outgoing message is about to be sent"""


class PosterityInstance(Component, ComponentManager):
    """Posterity instance directory
    
    A Posterity instance directory is a directory on a filesystem which holds
    information and settings for single Posterity installation.
    """
    VERSION_MSG = """Posterity instance directory version 1\n"""
    README_MSG = """This is a Posterity instance directory.\n""" \
    """Visit http://posterity.edgewall.org/ for more information.\n"""
    
    routes_providers = ExtensionPoint(IRoutesProvider)
    settings_panels = ExtensionPoint(ISettingsPanel)
    message_filters = ExtensionPoint(IMessageFilter)
    
    def __init__(self, path, create=False):
        super(PosterityInstance, self).__init__()
        if create:
            self._create(path)
        self._verify(path)
        self.path = path
        self.config = Config(os.path.join(path, 'conf', 'posterity.ini'))
        self.setup_logging()
        self.load_plugins()

    def load_plugins(self):
        for entry in iter_entry_points('posterity.plugins'):
            entry.load()

    def connect_db(self):
        """Initialize the SQLAlchemy database connection"""
        engine = create_engine(self.config.get('posterity', 'database'))
        metadata.connect(engine)
        return engine
    
    def run_in_transaction(self, callable):
        from posterity.db import transaction
        trans = objectstore.create_transaction()
        transaction._push_object(trans)
        try:
            try:
                objectstore.clear()
                callable()
                transaction.commit()
            except Exception, e:
                transaction.rollback()
                objectstore.clear()
                raise
        finally:
            transaction._pop_object(trans)
        

    def setup_logging(self):
        """Initialize the logging system for this instance.

        The logging system is controlled by a number of options under the 
        ``[logging]`` section of ``posterity.ini``:

        log_type
            Which backend to use. Available options: **file**, ``syslog`` and 
            ``errno``.
        log_file
            The file to log to if "log_type = file". This filename is relative
            to the instance directory.
        log_level
            Minumum level of importanance required for a message to be logged.
            Available values: ``CRITICAL``, ``ERROR``, ``WARNING``, **INFO**
            and ``DEBUG``.
        """
        self.log = logging.getLogger('posterity:' + self.path)
        log_type = self.config.get('logging', 'log_type', 'file').lower()
        if log_type == 'file':
            log_file = self.config.get('logging', 'log_file', 
                                       os.path.join('log', 'posterity.log'))
            handler = logging.FileHandler(os.path.join(self.path, log_file))
        elif log_type == 'syslog':
            handler = logging.handlers.SysLogHandler('/dev/log')
        elif log_type == 'stderr':
            handler = logging.StreamHandler()
        else:
            handler = logging.handlers.BufferingHandler(0)
        format = 'Posterity[%(module)s] %(levelname)s: %(message)s'
        formatter = logging.Formatter(format)
        handler.setFormatter(formatter)
        log_level = self.config.get('logging', 'log_level', 'INFO').upper()
        self.log.setLevel(log_level_map.get(log_level, logging.INFO))
        self.log.addHandler(handler)

    def _create(self, path):
        if os.path.exists(path) and len(os.listdir(path)) > 0:
            raise ValueError('Path %s already exists and is not empty' % path)
        os.mkdir(path)
        os.mkdir(os.path.join(path, 'conf'))
        os.mkdir(os.path.join(path, 'log'))
        os.mkdir(os.path.join(path, 'plugins'))
        fd = open(os.path.join(path, 'README.txt'), 'w')
        fd.write(self.README_MSG)
        fd = open(os.path.join(path, 'VERSION'), 'w')
        fd.write(self.VERSION_MSG)
        config = Config()
        config.set('smtp', 'server', 'localhost')
        config.set('logging', 'log_type', 'file')
        config.set('logging', 'log_file', os.path.join('log', 'posterity.log'))
        config.set('logging', 'log_level', 'INFO')
        config.write(os.path.join(path, 'conf', 'posterity.ini'))

    def _verify(self, path):
        ver_filename = os.path.join(path, 'VERSION')
        if not os.path.exists(ver_filename) or \
            open(ver_filename, 'r').read() != self.VERSION_MSG:
            raise ValueError('Path %s does not appear to be a posterity '
                             'instance directory' % path)
        

def make_posterity_app(config, **kw):
    app = PosterityApplication(config, **kw)
    return RegistryManager(app)


class PosterityApplication(object):
    """Posterity WSGI application"""
    
    def __init__(self, config, **kw):
        """Initialize the Posterity wsgi application
        
        :param config: The global configuration
        """
        super(PosterityApplication, self).__init__()
        config.update(kw)
        if not 'instance' in config:
            raise ValueError('You need to specify an instance= option')
        self.instance = config['instance']
        if isinstance(self.instance, basestring):
            self.instance = PosterityInstance(self.instance)
        app = self.setup_routes()
        app = CSRFMiddleware(app)
        app = SessionMiddleware(app)
        dburi = self.instance.config.get('posterity', 'database')
        app = TransactionMiddleware(app, dburi)
        base_url = self.instance.config.get('posterity', 'base_url')
        if base_url:
            app = PatchEnvironMiddleware(app, base_url)
        self.app = FormRewriteMiddleware(app)

    def __call__(self, environ, start_response):
        resp = Response()
        environ['paste.registry'].register(response, resp)
        req = Request(environ)
        environ['paste.registry'].register(app, self)
        environ['paste.registry'].register(log, self.instance.log)
        environ['paste.registry'].register(instance, self.instance)
        environ['paste.registry'].register(config, self.instance.config)
        environ['paste.registry'].register(request, req)
        setup_i18n(request)
        return self.app(environ, start_response)

    def setup_routes(self):
        routes = selector.Selector()
        for provider in self.instance.routes_providers:
            provider.add_routes(routes)
        chrome = make_pkg_resources({}, 'Posterity', 'posterity/static')
        routes.add('/', GET=self._redirect_to_inbox)
        routes.add('/chrome|', GET=chrome)
        return routes

    def _redirect_to_inbox(self, environ, start_response):
        redir = HTTPSeeOther.relative_redirect('/tags/Inbox/convs', environ)
        return redir.wsgi_application(environ, start_response)

