# -*- coding: utf-8 -*-
#
# Copyright (C) 2006-2007 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://posterity.edgewall.org/wiki/License.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://posterity.edgewall.org/log/.

from elixir import objectstore

from posterity.tests.fixtures import PosterityTestApp

def setup_module():
    global app
    app = PosterityTestApp()

def teardown_module():
    app.teardown_instance()

def test_login():
    resp = app.get('/', status=303).follow()
    form = resp.forms['login-form']
    form.set('username', 'testuser')
    form.set('password', 'incorrect')
    resp = form.submit()
    resp.mustcontain('Invalid username and/or password')
    form = resp.forms['login-form']
    form.set('username', 'testuser')
    form.set('password', 'secretpassword')
    resp = form.submit().follow(status=200)
    assert resp.request.url == '/tags/Inbox/convs'

def test_logout():
    app.get('/tags/Inbox/convs', status=200)
    app.get('/logout').follow(status=200)
    resp = app.get('/tags/Inbox/convs').follow(status=200)
    # Login again
    form = resp.forms['login-form']
    form.set('username', 'testuser')
    form.set('password', 'secretpassword')
    resp = form.submit().follow(status=200)

def test_add_tag():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp = resp.click('Settings')
    form = resp.forms['create-tag']
    form.set('name', 'My tag')
    resp = form.submit().follow(status=200)
    resp.mustcontain('Tag added', 'My tag')

def test_add_tag_name_collision():
    resp = app.get('/tags', status=200)
    form = resp.forms['create-tag']
    form.set('name', 'My tag')
    resp = form.submit(status=200)
    resp.mustcontain('This tag is already in use')
    
def test_edit_tag():
    resp = app.get('/tags/My%20tag', status=200)
    form = resp.forms['edit-tag']
    assert form['name'].value == 'My tag'
    form.set('name', 'Something else')
    resp = form.submit().follow(status=200)
    resp.mustcontain('Tag updated', 'Something else')

def test_edit_tag_name_collision():
    resp = app.get('/tags/Something%20else', status=200)
    form = resp.forms['edit-tag']
    form.set('name', 'Inbox')
    resp = form.submit(status=200)
    resp.mustcontain('This tag is already in use')
    
def test_delete_tag():
    resp = app.get('/tags/Something%20else', status=200)
    form = resp.forms['delete-tag']
    resp = form.submit().follow(status=200)
    resp.mustcontain('Tag deleted')
    assert not 'Something else' in resp

def test_add_identity():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp = resp.click('Settings')
    resp = resp.click('Identities')
    form = resp.forms['create-identity']
    form.set('name', 'My name')
    form.set('email', 'this is invalid')
    resp = form.submit(status=200)
    resp.mustcontain('An email address must contain a single @')
    form = resp.forms['create-identity']
    form.set('email', 'test@example.com')
    resp = form.submit().follow(status=200)
    resp.mustcontain('My name', 'Identity added')
    # Add a second identity
    form = resp.forms['create-identity']
    form.set('name', 'John Doe')
    form.set('email', 'john@example.com')
    resp = form.submit().follow(status=200)

def test_edit_identity():
    resp = app.get('/identities', status=200)
    resp = resp.click('My name')
    form = resp.forms['edit-identity']
    assert form['name'].value == 'My name'
    assert form['email'].value == 'test@example.com'
    form.set('name', 'My new name')
    form.set('email', 'test2@example.com')
    resp = form.submit().follow(status=200)
    resp.mustcontain('Identity updated', 'My new name')

def test_delete_identity():
    resp = app.get('/identities', status=200)
    resp = resp.click('My new name')
    form = resp.forms['delete-identity']
    resp = form.submit().follow(status=200)
    resp.mustcontain('Identity deleted')
    assert not 'My new name' in resp

def test_conv_index():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp.mustcontain('[Trac-dev] Developing a "superuser" simple plugin',
                     '[Trac-dev] Upload to repository via web?',
                     'Iradier Muro',
                     'Firstborn')

def test_conv_show():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp = resp.click('Developing a "superuser" simple plugin')
    resp.mustcontain('[Trac-dev] Developing a "superuser" simple plugin')

def test_reply_button():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp = resp.click('Developing a "superuser" simple plugin')
    resp = resp.clickbutton('Reply', index=0)
    resp.mustcontain('Composer')
    form = resp.forms['composer']
    assert form['to'].value == 'trac-dev@googlegroups.com'
    assert form['subject'].value == \
        'Re: [Trac-dev] Developing a &#34;superuser&#34; simple plugin'

def test_attachments():
    resp = app.get('/tags/Inbox/convs', status=200)
    resp = resp.click('A text/html test message')
    resp.mustcontain('posterity_logo.png')
    resp = resp.click('posterity_logo.png')
    assert resp.header('Content-Disposition') == \
        'attachment; filename="posterity_logo.png"'
    assert resp.header('Content-Length') == '9135'
