# -*- coding: utf-8 -*-
#
# Copyright (C) 2006-2007 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://posterity.edgewall.org/wiki/License.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://posterity.edgewall.org/log/.

import base64
import ConfigParser
from datetime import datetime, timedelta, tzinfo
import md5
import random
import sha
import time


class Config(ConfigParser.SafeConfigParser):
    """ConfigParser but with a sane api
    
    >>> config = Config()
    >>> config.set('foo', 'bar', 'boo')
    >>> config.get('foo', 'bar', 'boo')
    'boo'
    >>> config.get('foo', 'missing')

    >>> config.get('foo', 'missing', 'default_value')
    'default_value'
    """
    def __init__(self, filename=None):
        ConfigParser.SafeConfigParser.__init__(self)
        self._filename = filename
        if filename:
            self.read(filename)

    def write(self, filename=None):
        fd = open(filename or self._filename, 'w')
        ConfigParser.SafeConfigParser.write(self, fd)
    
    def set(self, section, param, value):
        """Set an option, new sections are added if missing"""
        if not self.has_section(section):
            self.add_section(section)
        ConfigParser.SafeConfigParser.set(self, section, param, value)

    def get(self, section, param, default=None):
        """Get an option value for a given section.
        
        `default` is returning if no option value exists"""
        try:
            return ConfigParser.SafeConfigParser.get(self, section, param)
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            return default


def encrypt_password(password):
    """Return a SHA1/base64 encrypted version of the password
    
    The output is compatible with the apache htpasswd utility.
    """
    return '{SHA}'+base64.encodestring(sha.sha(password).digest())[:-1]

def hex_entropy(bytes=32):
    """Returns a `bytes` characters long string containing random data"""
    return md5.md5(str(random.random())).hexdigest()[:bytes]


class FixedOffset(tzinfo):
    """Fixed offset in minutes east from UTC."""

    def __init__(self, offset, name):
        self._offset = timedelta(minutes=offset)
        self.zone = name

    def __str__(self):
        return self.zone

    def __repr__(self):
        return '<FixedOffset "%s" %s>' % (self.zone, self._offset)

    def utcoffset(self, dt):
        return self._offset

    def tzname(self, dt):
        return self.zone

    def dst(self, dt):
        return _zero


STDOFFSET = timedelta(seconds=-time.timezone)
if time.daylight:
    DSTOFFSET = timedelta(seconds=-time.altzone)
else:
    DSTOFFSET = STDOFFSET

DSTDIFF = DSTOFFSET - STDOFFSET


class LocalTimezone(tzinfo):
    """A 'local' time zone implementation"""

    def utcoffset(self, dt):
        if self._isdst(dt):
            return DSTOFFSET
        else:
            return STDOFFSET

    def dst(self, dt):
        if self._isdst(dt):
            return DSTDIFF
        else:
            return _zero

    def tzname(self, dt):
        return time.tzname[self._isdst(dt)]

    def _isdst(self, dt):
        tt = (dt.year, dt.month, dt.day,
              dt.hour, dt.minute, dt.second,
              dt.weekday(), 0, -1)
        try:
            stamp = time.mktime(tt)
            tt = time.localtime(stamp)
            return tt.tm_isdst > 0
        except OverflowError:
            return False


utc = FixedOffset(0, 'UTC')
utcmin = datetime.min.replace(tzinfo=utc)
utcmax = datetime.max.replace(tzinfo=utc)
_epoc = datetime(1970, 1, 1, tzinfo=utc)
_zero = timedelta(0)

localtz = LocalTimezone()

