# -*- coding: utf-8 -*-
#
# Copyright (C) 2006-2007 Edgewall Software
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://posterity.edgewall.org/wiki/License.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://posterity.edgewall.org/log/.

import poplib
import socket

import formencode

from posterity import log
from posterity.model import Tag, Identity


class TagNameValidator(formencode.validators.UnicodeString):
    """Tag name validator. Makes sure the name is not alreay used
    """
    messages = {'exists': 'This tag is already in use'}
    def validate_python(self, value, state):
        if Tag.get_by(account=state.account, name=value):
            raise formencode.Invalid(self.message('exists', state), 
                                     value, state)


class IdentityValidator(formencode.validators.FancyValidator):
    messages = {'invalid': 'Invalid identity id'}
    def _to_python(self, value, state):
        identity = Identity.get_by(account=state.account, id=int(value))
        if not identity:
            raise formencode.Invalid(self.message('invalid', state), 
                                     value, state)
        return identity


class TagValidator(formencode.validators.UnicodeString):
    messages = {'missing': 'Tag does not exist'}
    if_empty = None

    def _from_python(self, value, state):
        return value and value.name or ''
        
    def _to_python(self, value, state):
        tag = Tag.get_by(account=state.account, name=value)
        if not tag:
            raise formencode.Invalid(self.message('missing', state), 
                                     value, state)
        return tag


class TagSchema(formencode.Schema):
    name = TagNameValidator(not_empty=True)


class IdentitySchema(formencode.Schema):
    name = formencode.validators.UnicodeString(not_empty=True)
    email = formencode.validators.Email(not_empty=True)
    organization = formencode.validators.UnicodeString(if_missing='')
    reply_to = formencode.validators.UnicodeString(if_missing='')
    signature = formencode.validators.UnicodeString(if_missing='')


class ComposerSchema(formencode.Schema):
    identity = IdentityValidator(not_empty=True)
    to = formencode.validators.UnicodeString(not_empty=True)
    cc = formencode.validators.UnicodeString()
    subject = formencode.validators.UnicodeString(not_empty=True)
    body = formencode.validators.UnicodeString(not_empty=True)
    in_reply_to = formencode.validators.UnicodeString()
    references = formencode.validators.UnicodeString()


class FilterSchema(formencode.Schema):
    enabled = formencode.validators.Bool(if_missing=False)
    from_ = formencode.validators.UnicodeString(if_missing=None)
    to = formencode.validators.UnicodeString(if_missing=None)
    subject = formencode.validators.UnicodeString(if_missing=None)
    contains = formencode.validators.UnicodeString(if_missing=None)
    archive = formencode.validators.Bool(if_missing=False)
    delete = formencode.validators.Bool(if_missing=False)
    tag = TagValidator()
    forward_to = formencode.validators.Email(if_missing=None)


class Pop3Validator(formencode.validators.FancyValidator):
    """POP3 Credential validator
    
    This validator verifies two things:
    
    1. ``value['hostname']`` is a working pop3 server responding to connections
    2. It is possible to login using ``value['username']`` ``value['password']``
    
    This validator is supposed to be used as a "chained_validator"
    """
    messages = {'confailed': 'Connection test failed: %(err)s',
                'loginfailed': 'Login test failed, '
                               'check your username and password'}
        
    def validate_python(self, value_dict, state):
        try:
            if value_dict['hostname'].endswith('.example.com'):
                from posterity.tests.fixtures import MockPOP3
                server = MockPOP3(value_dict['hostname'])
            elif value_dict['use_ssl']:
                server = poplib.POP3_SSL(value_dict['hostname'])
            else:
                server = poplib.POP3(value_dict['hostname'])
            try:
                server.user(value_dict['username'])
                server.pass_(value_dict['password'])
            except poplib.error_proto:
                raise formencode.Invalid(self.message('loginfailed', state), 
                                         value_dict, state)
        except socket.error, e:
            raise formencode.Invalid(self.message('confailed', state, 
                                                  err=e.args[1]), 
                                     value_dict, state)
        except Exception, e:
            if isinstance(e, formencode.Invalid):
                raise
            log.exception(e)
            raise formencode.Invalid(self.message('confailed', state, 
                                                  err='Internal error'), 
                                     value_dict, state)


class Pop3AccountSchema(formencode.Schema):
    enabled = formencode.validators.Bool(if_missing=False)
    hostname = formencode.validators.UnicodeString(min=1)
    username = formencode.validators.UnicodeString()
    password = formencode.validators.UnicodeString()
    use_ssl = formencode.validators.Bool(if_missing=False)
    keep_time = formencode.validators.Number()
    archive = formencode.validators.Bool(if_missing=False)
    tag = TagValidator()
    chained_validators = [Pop3Validator()]